(ns dda.c4k-keycloak.keycloak-test
  (:require
   #?(:clj [clojure.test :refer [deftest is are testing run-tests]]
      :cljs [cljs.test :refer-macros [deftest is are testing run-tests]])
   [dda.c4k-keycloak.keycloak :as cut]))

(deftest should-generate-secret
  (is (= {:apiVersion "v1"
          :kind "Secret"
          :metadata {:name "keycloak-secret"}
          :type "Opaque"
          :data
          {:keycloak-user "dXNlcg=="
           :keycloak-password "cGFzc3dvcmQ="}}
         (cut/generate-secret {:keycloak-admin-user "user" :keycloak-admin-password "password"}))))

(deftest should-generate-certificate
  (is (= {:apiVersion "cert-manager.io/v1alpha2"
          :kind "Certificate"
          :metadata {:name "keycloak-cert", :namespace "default"}
          :spec
          {:secretName "keycloak-secret"
           :commonName "test.de"
           :dnsNames ["test.de"]
           :issuerRef {:name "letsencrypt-prod-issuer", :kind "ClusterIssuer"}}}
         (cut/generate-certificate {:fqdn "test.de" :issuer :prod} ))))

(deftest should-generate-ingress-yaml-with-default-issuer
  (is (= {:apiVersion "networking.k8s.io/v1beta1"
           :kind "Ingress"
           :metadata
           {:name "ingress-cloud"
            :annotations
            {:cert-manager.io/cluster-issuer "letsencrypt-staging-issuer"
             :nginx.ingress.kubernetes.io/proxy-body-size "256m"
             :nginx.ingress.kubernetes.io/ssl-redirect "true"
             :nginx.ingress.kubernetes.io/rewrite-target "/"
             :nginx.ingress.kubernetes.io/proxy-connect-timeout "300"
             :nginx.ingress.kubernetes.io/proxy-send-timeout "300"
             :nginx.ingress.kubernetes.io/proxy-read-timeout "300"}
            :namespace "default"}
           :spec
           {:tls [{:hosts ["test.de"] :secretName "keycloak-secret"}]
            :rules [{:host "test.de", :http {:paths [{:backend {:serviceName "keycloak", :servicePort 8080}}]}}]}}
          (cut/generate-ingress {:fqdn "test.de"}))))

(deftest should-generate-ingress-yaml-with-prod-issuer
  (is (= {:apiVersion "networking.k8s.io/v1beta1"
          :kind "Ingress"
          :metadata
          {:name "ingress-cloud"
           :annotations
           {:cert-manager.io/cluster-issuer "letsencrypt-prod-issuer"
            :nginx.ingress.kubernetes.io/proxy-body-size "256m"
            :nginx.ingress.kubernetes.io/ssl-redirect "true"
            :nginx.ingress.kubernetes.io/rewrite-target "/"
            :nginx.ingress.kubernetes.io/proxy-connect-timeout "300"
            :nginx.ingress.kubernetes.io/proxy-send-timeout "300"
            :nginx.ingress.kubernetes.io/proxy-read-timeout "300"}
           :namespace "default"}
          :spec
          {:tls [{:hosts ["test.de"], :secretName "keycloak-secret"}]
           :rules '({:host "test.de", :http {:paths [{:backend {:serviceName "keycloak", :servicePort 8080}}]}})}}
         (cut/generate-ingress {:fqdn "test.de"
                                :issuer :prod}))))

(deftest should-generate-deployment
  (is (= {:apiVersion "apps/v1"
          :kind "Deployment"
          :metadata
          {:name "keycloak"
           :namespace "default"
           :labels {:app "keycloak"}}
          :spec
          {:replicas 1
           :selector {:matchLabels {:app "keycloak"}}
           :template
           {:metadata {:labels {:app "keycloak"}}
            :spec
            {:containers
             [{:name "keycloak"
               :image "quay.io/keycloak/keycloak:13.0.0"
               :env
               [{:name "DB_VENDOR", :value "POSTGRES"}
                {:name "DB_ADDR", :value "postgresql-service"}
                {:name "DB_SCHEMA", :value "public"}
                {:name "DB_DATABASE"
                 :valueFrom
                 {:configMapKeyRef
                  {:name "postgres-config", :key "postgres-db"}}}
                {:name "DB_USER"
                 :valueFrom
                 {:secretKeyRef
                  {:name "postgres-secret", :key "postgres-user"}}}
                {:name "DB_PASSWORD"
                 :valueFrom
                 {:secretKeyRef
                  {:name "postgres-secret"
                   :key "postgres-password"}}}
                {:name "PROXY_ADDRESS_FORWARDING", :value "true"}
                {:name "KEYCLOAK_USER"
                 :valueFrom
                 {:secretKeyRef
                  {:name "keycloak-secret", :key "keycloak-user"}}}
                {:name "KEYCLOAK_PASSWORD"
                 :valueFrom
                 {:secretKeyRef
                  {:name "keycloak-secret"
                   :key "keycloak-password"}}}]
               :ports [{:name "http", :containerPort 8080}]
               :readinessProbe
               {:httpGet
                {:path "/auth/realms/master", :port 8080}}}]}}}}
         (cut/generate-deployment))))
